@@ -2,14 +2,15 @@
 
 import os
 import sys
+import ast
 import traceback
 from argparse import ArgumentParser
 import numpy as np
 import onnx
 import onnx_graphsurgeon
 import onnx_graphsurgeon as gs
-from onnx_graphsurgeon.ir.tensor import Variable
-from typing import Tuple, Optional, List
+from onnx_graphsurgeon.ir.tensor import Variable, Constant
+from typing import Optional, List
 
 class Color:
     BLACK          = '\033[30m'
@@ -66,7 +67,8 @@ CONSTANT_DTYPES_TO_NUMPY_TYPES = {
 
 def __search_op_constant_from_input_constant_name(
     graph: onnx_graphsurgeon.Graph,
-    input_constant_name: str
+    input_constant_name: str,
+    op_name: str,
 ):
     """
     Parameters
@@ -77,22 +79,57 @@ def __search_op_constant_from_input_constant_name(
     input_constant_name: str
         input_constant_name of the search target.
 
+    op_name: str
+        op_name of the search target.
+
     Returns
     -------
-    input_constant_to_change:
+    input_constant_to_change: gs.Node
         constant found.
         If not found, return an None.
+
+    new_constant_name: str
+        A new name to be given only to the constant
+        with the corresponding op_name if there are
+        multiple constants with the same name in the model.
     """
     # Search for variable matching variable_name
     input_constant_to_change = None
-    for graph_node in graph.nodes:
+    graph_nodes = None
+    if op_name:
+        graph_nodes = [graph_node for graph_node in graph.nodes if graph_node.name == op_name]
+    else:
+        graph_nodes = graph.nodes
+
+    for graph_node in graph_nodes:
         for input in graph_node.inputs:
-            if isinstance(input, Variable) and input.name == input_constant_name:
-                input_constant_to_change = input.inputs[0]
+            if isinstance(input, Constant) and input.name == input_constant_name:
+                input_constant_to_change = input
                 break
+            elif isinstance(input, Variable) and input.name == input_constant_name:
+                if input.inputs[0].op == 'Constant':
+                    input_constant_to_change = input.inputs[0]
+                    break
+        else:
+            continue
+        break
+
+    # If the same input_constant_name constant exists in the model,
+    # a new name is given only to the constant with the corresponding op_name.
+    new_constant_name = None
+    if input_constant_to_change is not None and op_name:
+        num_of_const_with_the_same_name = sum(
+            [
+                1 for graph_node in graph.nodes \
+                    for input in graph_node.inputs \
+                        if input.name == input_constant_name
+            ]
+        )
+        if num_of_const_with_the_same_name > 1:
+            new_constant_name = f'{input_constant_name}_mod_{num_of_const_with_the_same_name}'
 
     # Return variable
-    return input_constant_to_change
+    return input_constant_to_change, new_constant_name
 
 
 def modify(
@@ -101,9 +138,10 @@ def modify(
     onnx_graph: Optional[onnx.ModelProto] = None,
     op_name: Optional[str] = '',
     attributes: Optional[dict] = None,
+    delete_attributes: Optional[List[str]] = None,
     input_constants: Optional[dict] = None,
     non_verbose: Optional[bool] = False,
-) -> Tuple[onnx.ModelProto, List[str]]:
+) -> onnx.ModelProto:
 
     """
     Parameters
@@ -141,6 +179,15 @@ def modify(
         Default: None\n\
         https://github.com/onnx/onnx/blob/main/docs/Operators.md
 
+    delete_attributes: Optional[List[str]]
+        Parameter to delete the attribute of the OP specified in --op_name.\n\
+        If the OP specified in --op_name has no attributes, it is ignored.\n\
+        delete_attributes can be specified multiple times.\n\
+        --delete_attributes name1 name2 name3\n\
+        https://github.com/onnx/onnx/blob/main/docs/Operators.md\n\n\
+        e.g.\n\
+        --delete_attributes alpha beta
+
     input_constants: Optional[dict]
         Specifies the name of the constant to be changed. \n\
         If you want to change only the constant, \n\
@@ -196,11 +243,22 @@ def modify(
     # Updating Attributes
     # attributes = {"alpha": 1.0, "beta": 1.0, "transA": 0, "transB": 0}
     if node_subject_to_change:
-        for node_subject_to_change_attr_key in node_subject_to_change.attrs.keys():
+        # Update
+        if attributes:
             for update_attr_key, update_attr_value in attributes.items():
-                if node_subject_to_change_attr_key == update_attr_key:
-                    node_subject_to_change.attrs[node_subject_to_change_attr_key] = update_attr_value
-                    break
+                found_flg = False
+                for node_subject_to_change_attr_key in node_subject_to_change.attrs.keys():
+                    if node_subject_to_change_attr_key == update_attr_key:
+                        node_subject_to_change.attrs[node_subject_to_change_attr_key] = update_attr_value
+                        found_flg = True
+                        break
+                if not found_flg:
+                    node_subject_to_change.attrs[update_attr_key] = update_attr_value
+
+        # Delete
+        if delete_attributes:
+            node_subject_to_change.attrs = \
+                {attr_key: attr_value for attr_key, attr_value in node_subject_to_change.attrs.items() if attr_key not in delete_attributes}
 
     # Updating Constants
     """
@@ -226,11 +284,57 @@ def modify(
     """
     if input_constants:
         for input_constant_name, input_constant_value in input_constants.items():
-            constant = __search_op_constant_from_input_constant_name(graph, input_constant_name)
-            constant.attrs['value'] = gs.Constant(
-                name='',
-                values=input_constant_value
+            constant, new_constant_name = __search_op_constant_from_input_constant_name(
+                graph,
+                input_constant_name,
+                op_name,
             )
+            if constant is not None:
+                if hasattr(constant, "attrs"):
+                    constant.attrs['value'] = gs.Constant(
+                        name='' if new_constant_name is None else new_constant_name,
+                        values=input_constant_value
+                    )
+                else:
+                    if new_constant_name is None:
+                        constant.values = input_constant_value
+                    else:
+                        if op_name:
+                            new_constant = gs.Constant(
+                                name=new_constant_name,
+                                values=input_constant_value
+                            )
+                            graph_nodes = [graph_node for graph_node in graph.nodes if graph_node.name == op_name]
+                            target_node = None
+                            target_input_idx = -1
+                            for graph_node in graph_nodes:
+                                for idx, graph_node_input in enumerate(graph_node.inputs):
+                                    if graph_node_input.name == input_constant_name:
+                                        target_node = graph_node
+                                        target_input_idx = idx
+                                        break
+                                else:
+                                    continue
+                                break
+                            if target_node and target_input_idx > -1:
+                                target_node.inputs[target_input_idx] = new_constant
+                            else:
+                                pass
+                        else:
+                            constant.values = input_constant_value
+
+            else:
+                if not non_verbose:
+                    if op_name:
+                        print(
+                            f'{Color.YELLOW}WARNING:{Color.RESET} '+
+                            f'op_name: {op_name}, input_constant_name: {input_constant_name} did not exist in the model.'
+                        )
+                    else:
+                        print(
+                            f'{Color.YELLOW}WARNING:{Color.RESET} '+
+                            f'input_constant_name: {input_constant_name} did not exist in the model.'
+                        )
 
     # Cleanup
     graph.cleanup().toposort()
@@ -289,7 +393,6 @@ def main():
         action='append',
         help=\
             'Parameter to change the attribute of the OP specified in --op_name. \n'+
-            'If the OP specified in --op_name has no attributes, it is ignored. \n'+
             'attributes can be specified multiple times. \n'+
             '--attributes name dtype value \n'+
             'dtype is one of "float32" or "float64" or "int32" or "int64" or "str". \n'+
@@ -300,6 +403,18 @@ def main():
             '--attributes transA int64 0 \n'+
             '--attributes transB int64 0'
     )
+    parser.add_argument(
+        '--delete_attributes',
+        nargs='+',
+        help=\
+            'Parameter to delete the attribute of the OP specified in --op_name. \n'+
+            'If the OP specified in --op_name has no attributes, it is ignored. \n'+
+            'delete_attributes can be specified multiple times. \n'+
+            '--delete_attributes name1 name2 name3 \n'+
+            'https://github.com/onnx/onnx/blob/main/docs/Operators.md \n\n'+
+            'e.g.\n'+
+            '--delete_attributes alpha beta'
+    )
     parser.add_argument(
         '--input_constants',
         type=str,
@@ -327,6 +442,7 @@ def main():
     input_constants = args.input_constants
     op_name = args.op_name
     attributes = args.attributes
+    delete_attributes = args.delete_attributes
     non_verbose = args.non_verbose
 
     # file existence check
@@ -345,7 +461,7 @@ def main():
     graph = gs.import_onnx(onnx_graph)
 
     # op_name and attributes must always be specified at the same time.
-    if (op_name and not attributes) or (not op_name and attributes):
+    if (not op_name and attributes) or (not op_name and delete_attributes):
         print(
             f'{Color.RED}ERROR:{Color.RESET} '+
             f'op_name and attributes must always be specified at the same time.'
@@ -363,7 +479,12 @@ def main():
             # parse
             attr_name = attribute[0]
             attr_type = attribute[1]
-            attr_value = eval(attribute[2])
+            if attr_type == 'string':
+                attr_type = 'str'
+            if attr_type != 'str':
+                attr_value = ast.literal_eval(attribute[2])
+            else:
+                attr_value = attribute[2]
 
             # dtype check
             if attr_type not in ATTRIBUTE_DTYPES_TO_NUMPY_TYPES:
@@ -387,7 +508,7 @@ def main():
         input_constants_tmp = {}
         for input_constant in input_constants:
             # Search for OPs corresponding to the name of input_constant
-            constant = __search_op_constant_from_input_constant_name(graph, input_constant[0])
+            constant, _ = __search_op_constant_from_input_constant_name(graph, input_constant[0], op_name)
 
             # None: Not found
             if not constant:
@@ -400,7 +521,7 @@ def main():
             # Parse
             constant_name = input_constant[0]
             constant_type = input_constant[1]
-            constant_value = eval(input_constant[2])
+            constant_value = ast.literal_eval(input_constant[2])
 
             # dtype check
             if constant_type not in CONSTANT_DTYPES_TO_NUMPY_TYPES:
@@ -423,6 +544,7 @@ def main():
         onnx_graph=onnx_graph,
         op_name=op_name,
         attributes=attributes_tmp,
+        delete_attributes=delete_attributes,
         input_constants=input_constants_tmp,
         non_verbose=non_verbose
     )
